@@ -15,6 +15,8 @@ GoodFixtures = {
   "test_v5header.bmp"=>[:bmp, [40, 27]],
   "test.gif"=>[:gif, [17, 32]],
   "animated.gif"=>[:gif, [400, 400]],
+  "animated.png"=>[:png, [100, 100]],
+  "animated_without_gct.gif"=>[:gif, [859, 478]],
   "test.jpg"=>[:jpeg, [882, 470]],
   "test.png"=>[:png, [30, 20]],
   "test2.jpg"=>[:jpeg, [250, 188]],
@@ -33,12 +35,32 @@ GoodFixtures = {
   "webp_vp8x.webp" => [:webp, [386, 395]],
   "webp_vp8l.webp" => [:webp, [386, 395]],
   "webp_vp8.webp" => [:webp, [550, 368]],
+  "webp_animated.webp" => [:webp, [400, 400]],
   "test.svg" => [:svg, [200, 300]],
   "test_partial_viewport.svg" => [:svg, [860, 400]],
   "test2.svg" => [:svg, [366, 271]],
   "test3.svg" => [:svg, [255, 48]],
   "test4.svg" => [:svg, [271, 271]],
-  "orient_6.jpg"=>[:jpeg, [1250,2500]]
+  "test5.svg" => [:svg, [255, 48]],
+  "test7.svg" => [:svg, [100, 100]],
+  "orient_6.jpg"=>[:jpeg, [1250,2500]],
+  "heic/test.heic"=>[:heic, [700,476]],
+  "heic/heic-empty.heic"=>[:heic, [3992,2992]],
+  "heic/heic-iphone.heic"=>[:heic,[4032,3024]],
+  "heic/heic-iphone7.heic"=>[:heic,[4032,3024]],
+  "heic/heic-maybebroken.HEIC"=>[:heic,[4032,3024]],
+  "heic/heic-single.heic"=>[:heif,[1440,960]],
+  "heic/heic-collection.heic"=>[:heif,[1440,960]],
+  "heic/inverted.heic"=>[:heic,[3024, 4032]],
+  "heic/test-meta-after-mdat.heic"=>[:heic,[4000, 3000]],
+  "test6.svg" => [:svg, [450, 450]],
+  "avif/hato.avif" => [:avif, [3082, 2048]],
+  "avif/fox.avif" => [:avif, [1204, 799]],
+  "avif/kimono.avif" => [:avif, [722, 1024]],
+  "avif/red_green_flash.avif" => [:avif, [256, 256]],
+  "isobmff.jxl" => [:jxl, [1280,1600]],
+  "naked.jxl" => [:jxl, [1000,1000]],
+  "test.dng" => [:tiff, [4032, 3024]]
 }
 
 BadFixtures = [
@@ -47,7 +69,8 @@ BadFixtures = [
   "test.xml",
   "test2.xml",
   "a.CR2",
-  "a.CRW"
+  "a.CRW",
+  "avif/star.avifs",
 ]
 # man.ico courtesy of http://www.iconseeker.com/search-icon/artists-valley-sample/business-man-blue.html
 # test_rgb.ct courtesy of http://fileformats.archiveteam.org/wiki/Scitex_CT
@@ -92,22 +115,37 @@ end
 class FastImageTest < Test::Unit::TestCase
   def test_should_report_type_correctly
     GoodFixtures.each do |fn, info|
-      assert_equal info[0], FastImage.type(TestUrl + fn)
-      assert_equal info[0], FastImage.type(TestUrl + fn, :raise_on_failure=>true)
+      assert_equal info[0], FastImage.type(TestUrl + fn), "type of image #{fn} must be #{info[0]}"
+      assert_equal info[0], FastImage.type(TestUrl + fn, :raise_on_failure=>true), "type of image #{fn} must be #{info[0]}"
     end
   end
 
   def test_should_report_size_correctly
     GoodFixtures.each do |fn, info|
-      assert_equal info[1], FastImage.size(TestUrl + fn)
-      assert_equal info[1], FastImage.size(TestUrl + fn, :raise_on_failure=>true)
+      assert_equal info[1], FastImage.size(TestUrl + fn), "size for #{fn} must be #{info[1]}"
+      assert_equal info[1], FastImage.size(TestUrl + fn, :raise_on_failure=>true), "size for #{fn} must be #{info[1]}"
     end
   end
 
   def test_should_report_animated_correctly
-    assert_equal nil, FastImage.animated?(TestUrl + "test.png")
+    assert_equal nil, FastImage.animated?(TestUrl + "test.jpg")
+    assert_equal false, FastImage.animated?(TestUrl + "test.png")
     assert_equal false, FastImage.animated?(TestUrl + "test.gif")
     assert_equal true, FastImage.animated?(TestUrl + "animated.gif")
+    assert_equal true, FastImage.animated?(TestUrl + "animated.png")
+    assert_equal true, FastImage.animated?(TestUrl + "animated_without_gct.gif")
+    assert_equal false, FastImage.animated?(TestUrl + "webp_vp8x.webp")
+    assert_equal true, FastImage.animated?(TestUrl + "webp_animated.webp")
+    assert_equal false, FastImage.animated?(TestUrl + "avif/hato.avif")
+    assert_equal true, FastImage.animated?(TestUrl + "avif/red_green_flash.avif")
+  end
+
+  def test_should_report_multiple_properties
+    fi = FastImage.new(File.join(FixturePath, "animated.gif"))
+    assert_equal :gif, fi.type
+    assert_equal [400, 400], fi.size
+    assert_equal true, fi.animated
+    assert_equal 1001718, fi.content_length
   end
 
   def test_should_return_nil_on_fetch_failure
@@ -168,13 +206,13 @@ class FastImageTest < Test::Unit::TestCase
 
   def test_should_report_type_correctly_for_local_files
     GoodFixtures.each do |fn, info|
-      assert_equal info[0], FastImage.type(File.join(FixturePath, fn))
+      assert_equal info[0], FastImage.type(File.join(FixturePath, fn)), "type of image #{fn} must be #{info[0]}"
     end
   end
 
   def test_should_report_size_correctly_for_local_files
     GoodFixtures.each do |fn, info|
-      assert_equal info[1], FastImage.size(File.join(FixturePath, fn))
+      assert_equal info[1], FastImage.size(File.join(FixturePath, fn)), "size for #{fn} must be #{info[1]}"
     end
   end
 
@@ -185,7 +223,7 @@ class FastImageTest < Test::Unit::TestCase
   def test_should_report_type_correctly_for_ios
     GoodFixtures.each do |fn, info|
       File.open(File.join(FixturePath, fn), "r") do |io|
-        assert_equal info[0], FastImage.type(io)
+        assert_equal info[0], FastImage.type(io), "type of image #{fn} must be #{info[0]}"
       end
     end
   end
@@ -193,7 +231,7 @@ class FastImageTest < Test::Unit::TestCase
   def test_should_report_size_correctly_for_ios
     GoodFixtures.each do |fn, info|
       File.open(File.join(FixturePath, fn), "r") do |io|
-        assert_equal info[1], FastImage.size(io)
+        assert_equal info[1], FastImage.size(io), "size for #{fn} must be #{info[1]}"
       end
     end
   end
@@ -202,7 +240,7 @@ class FastImageTest < Test::Unit::TestCase
     GoodFixtures.each do |fn, info|
       File.open(File.join(FixturePath, fn), "r") do |io|
         io.read
-        assert_equal info[0], FastImage.type(io)
+        assert_equal info[0], FastImage.type(io), "type of image #{fn} must be #{info[0]}"
       end
     end
   end
@@ -211,7 +249,7 @@ class FastImageTest < Test::Unit::TestCase
     GoodFixtures.each do |fn, info|
       File.open(File.join(FixturePath, fn), "r") do |io|
         io.read
-        assert_equal info[1], FastImage.size(io)
+        assert_equal info[1], FastImage.size(io), "size for #{fn} must be #{info[1]}"
       end
     end
   end
@@ -408,6 +446,13 @@ class FastImageTest < Test::Unit::TestCase
     FakeWeb.register_uri(:get, url, :body => File.join(FixturePath, "test.jpg"), :content_length => 52)
 
     assert_equal 52, FastImage.new(url).content_length
+    
+    assert_equal 322, FastImage.new(File.join(FixturePath, "test.png")).content_length
+    assert_equal 322, FastImage.new(Pathname.new(File.join(FixturePath, "test.png"))).content_length
+
+    string = File.read(File.join(FixturePath, "test.png"))
+    stringio = StringIO.new(string)
+    assert_equal 322, FastImage.new(stringio).content_length
   end
 
   def test_content_length_not_provided
@@ -444,7 +489,7 @@ class FastImageTest < Test::Unit::TestCase
   def test_should_support_data_uri_scheme_images
     assert_equal DataUriImageInfo[0], FastImage.type(DataUriImage)
     assert_equal DataUriImageInfo[1], FastImage.size(DataUriImage)
-    assert_raises(FastImage::ImageFetchFailure) do
+    assert_raises(FastImage::CannotParseImage) do
       FastImage.type("data:", :raise_on_failure => true)
     end
   end
@@ -465,4 +510,35 @@ class FastImageTest < Test::Unit::TestCase
       FastImage.size(TestUrl + "a.CRW", :raise_on_failure=>true)
     end
   end
+  
+  def test_returns_nil_when_uri_is_nil
+    assert_equal nil, FastImage.size(nil)
+  end
+  
+  def test_raises_when_uri_is_nil_and_raise_on_failure_is_set
+    assert_raises(FastImage::BadImageURI) do
+      FastImage.size(nil, :raise_on_failure => true)
+    end
+  end
+  
+  def test_width
+    assert_equal 30, FastImage.new(TestUrl + "test.png").width
+    assert_equal nil, FastImage.new(TestUrl + "does_not_exist").width
+  end
+
+  def test_height
+    assert_equal 20, FastImage.new(TestUrl + "test.png").height
+    assert_equal nil, FastImage.new(TestUrl + "does_not_exist").height
+  end
+  
+  def test_content_length_after_size
+    fi = FastImage.new(File.join(FixturePath, "test.png"))
+    fi.size
+    assert_equal 322, fi.content_length
+  end
+  
+  def test_unknown_protocol
+    FakeWeb.register_uri(:get, "http://example.com/test", body: "", location: "hhttp://example.com", :status => 301)
+    assert_nil FastImage.size("http://example.com/test")
+  end
 end
